#include <math.h>
#include <array>
#include <complex>

inline std::complex<double> faddeeva(std::complex<double> z) {
	using namespace std::complex_literals;

	constexpr std::size_t M = 4;
	constexpr std::size_t N = 1 << (M-1);

	// Precomputed Faddeeva function approximation coefficients (M==4)
	constexpr std::array<std::complex<double>, N> A = {
		+0.983046454995208,
		-0.095450491368505,
		-0.106397537035019,
		+0.004553979597404,
		-0.000012773721299,
		-0.000000071458742,
		+0.000000000080803,
		-0.000000000000007,
	};
	constexpr std::array<std::complex<double>, N> B = {
		std::complex<double>{ 0.0, -1.338045597353875 },
		std::complex<double>{ 0.0, +0.822618936152688 },
		std::complex<double>{ 0.0, -0.044470795125534 },
		std::complex<double>{ 0.0, -0.000502542048995 },
		std::complex<double>{ 0.0, +0.000011914499129 },
		std::complex<double>{ 0.0, -0.000000020157171 },
		std::complex<double>{ 0.0, -0.000000000001558 },
		std::complex<double>{ 0.0, +0.000000000000003 },
	};
	constexpr std::array<std::complex<double>, N> C = {
		0.392699081698724,
		1.178097245096172,
		1.963495408493621,
		2.748893571891069,
		3.534291735288517,
		4.319689898685965,
		5.105088062083414,
		5.890486225480862,
	};
	constexpr double s = 2.75;

	// Constrain to imag(z)>=0
	const double sgni = std::imag(z)<0 ? -1 : 0;
	z *= sgni*2 + 1;

	// Approximate
	const auto t = z + 0.5i*s;
	std::complex<double> w = .0;
	for (std::size_t m=0; m<N; ++m)
		w += (A[m] + t*B[m]) / (C[m]*C[m] - t*t);

	return -sgni*(2.0*std::exp(-z*z)) + (sgni*2 + 1) * w;
}

inline std::complex<double> erf(std::complex<double> z) {
	using namespace std::complex_literals;
	return 1.0 - std::exp(-z*z) * faddeeva(1i*z);
}
