
disp([datestr(now, 'HH:MM:SS'),': Start...']);

subindex = @(V, i) V(i);
subindexm = @(V, i, j) V(i,j);

syms h k K y


syms a(h) b(h) c(h) ee(h) sigma_e(h)
syms c0 c1 c2 c3 c4

type = 'slab';

%$$ INPUT

% Expansion order
Order = 7;
Order2 = Order;
ProfileVersion = 0;

% % "Eval slab" material
a(h) = (h/3+0.5)*sin(c4);
c(h) = (h/3+0.5)*cos(c4);
b(h) = simplify(sqrt(1-a(h)^2-c(h)^2));
eo = 1.33;
ee(h) = 1.3 + h^2/5;
sigma_o = 0;
sigma_e(h) = h/4;
type = 'slab';

% % "E44 Liquid Crystal Plate - heat conduction" material
% Heat_ee_T15C = 1.81;
% Heat_ee_T30C = 1.79;
% syms T Heat_ee(T)
% Heat_ee(T) = simplify(Heat_ee_T30C*(T-15)/15 + Heat_ee_T15C*(1 - (T-15)/15));
% Heat_eo = 1.53;
% 
% T_start = 5;
% T_end   = 95;
% T_pow   = c2;
% Heat_stretch = 0.5;
% 
% a(h) = Heat_stretch*sin(c4);
% b(h) = sqrt(1 - Heat_stretch^2);
% c(h) = Heat_stretch*cos(c4);
% eo = Heat_eo;
% ee(h) = simplify(Heat_ee(T_start + (T_end-T_start)*(1-h/c3)^T_pow));
% sigma_o = 0;
% sigma_e(h) = 0;
% type = 'heat-conduction';

% % "5CB Liquid Crystal Radial" material
% a(h) = 0;
% b(h) = cos(c3+c4*h);
% c(h) = sin(c3+c4*h);
% eo = 1.534;
% ee(h) = 1.707;
% sigma_o = 0;
% sigma_e(h) = 0;
% type = 'droplet';

% % "E12 Liquid Crystal Radial" material
% a(h) = 0;
% b(h) = cos(c3+c4*h);
% c(h) = sin(c3+c4*h);
% eo = 1.52;
% ee(h) = 1.74;
% sigma_o = 0;
% sigma_e(h) = 0;
% type = 'droplet';

% % "E12 Liquid Crystal pi-wall-defect" material
% E12_stretch = 0.61;
% a(h) = sqrt(1-E12_stretch^2);
% b(h) = E12_stretch*cos(c3+c4*h);
% c(h) = E12_stretch*sin(c3+c4*h);
% eo = 1.52;
% ee(h) = 1.74;
% sigma_o = 0;
% sigma_e(h) = 0;
% type = 'droplet-pi-wall';

StoreOutputAt = "/tmp/gen/";

%$$ /INPUT

mkdir(StoreOutputAt);
workers=Inf;

temp  = sym('temp', [16 1]);
temp2 = sym('temp2', [16 1]);
tempv = sym('tempv', [6 3]);


%%% STAGE: Generating operator A
disp([datestr(now, 'HH:MM:SS'),': Electric fields and poynting vectors...']);

syms A(h) q_o q_e(h) ef(h) Wo(h) We(h) o(h) e(h) so(h) se(h) o(h) e(h) so(h) se(h)

assume(h, 'real');
assumeAlso(c0, 'real');
assumeAlso(c1, 'real');
assumeAlso(c2, 'real');
assumeAlso(c3, 'real');
assumeAlso(c4, 'real');
assumeAlso(K, 'real');
assumeAlso(k, 'real');
assumeAlso(a(h), 'real');
assumeAlso(b(h), 'real');
assumeAlso(c(h), 'real');
assumeAlso(ee(h), 'real');
assumeAlso(sigma_e(h), 'real');
assumeAlso(K>=0);
assumeAlso(k>0);
assumeAlso(h>0);
assumeAlso(ee(h)>0);

A(h) = simplify([a(h),b(h),c(h)]);
assumeAlso(A(h), 'real');

[~, q_e(h)] = qe(eo,ee(h),A(h),K);
q_e(h) = simplify(q_e(h));
q_o = -qo(eo,K);

ef(h) = sqrt(q_e(h)^2 + K^2);
Wo(h) = [0,q_o,-K];
We(h) = [0,q_e(h),-K];
AWe(h) = subindex(A(h),1)*subindex(We(h),1) + subindex(A(h),2)*subindex(We(h),2) + subindex(A(h),3)*subindex(We(h),3);


parfor (i=1:5, workers)
    assume([in([c0, c1, c2, c3, c4, h, A(h), K, k, a(h), b(h), c(h), ee(h), sigma_e(h)], 'real'), K>=0, k>0, h>0, ee(h)>0]);
    
    if     i==1; tempv(i,:) = simplify(eo^2*A(h) - AWe(h)*We(h));
    elseif i==2; temp(i)    = (combine(eo^4 + AWe(h)^2*(q_e(h)^2 + K^2 - 2*eo^2)));
    elseif i==3; tempv(i,:) = simplify(cross(A(h),Wo(h)) / sqrt(eo^2 - dot(A(h),Wo(h))^2));
    elseif i==4; tempv(i,:) = simplify(Wo(h)/sqrt(q_o^2+K^2));
    elseif i==5; tempv(i,:) = simplify(We(h)/sqrt(q_e(h)^2+K^2));
    end
%     disp([datestr(now, 'HH:MM:SS'),': ', num2str(i), '/6']);
end
e(h) = tempv(1,:) / sqrt(temp(2));
o(h) = tempv(3,:);
so(h) = tempv(4,:);
se(h) = tempv(5,:);

ox(h)  = subindex(o(h), 1);
oy(h)  = subindex(o(h), 2);
oz(h)  = subindex(o(h), 3);
ex(h)  = subindex(e(h), 1);
ey(h)  = subindex(e(h), 2);
ez(h)  = subindex(e(h), 3);
soz(h) = subindex(so(h),3);
soy(h) = subindex(so(h),2);
sez(h) = subindex(se(h),3);
sey(h) = subindex(se(h),2);

disp([datestr(now, 'HH:MM:SS'),': Generating operator A...']);

syms Dox(h) Doz(h) Dex(h) Dez(h) dpo(h) dpe(h) Dpe(h) tau_e(h) tau_o(h) denom(h) Ar00(h) Ar01(h) Ar10(h) Ar11(h)
Dox(h) = diff(ox,h);
Doz(h) = diff(oz,h);
Dex(h) = diff(ex,h);
Dez(h) = diff(ez,h);

parfor (i=1:10, workers)
    assume([in([c0, c1, c2, c3, c4, h, A(h), K, k, a(h), b(h), c(h), ee(h), sigma_e(h)], 'real'), K>=0, k>0, h>0, ee(h)>0]);
    
    if     i==1; temp(i) = simplifyFraction(eo   /soy(h));
    elseif i==2; temp(i) = simplifyFraction(ef(h)/sey(h));
    elseif i==3; temp(i) = K*simplifyFraction(sez(h)/sey(h) - soz(h)/soy(h));
    elseif i==4; temp(i) = -simplifyFraction(sigma_o   /soy(h));
    elseif i==5; temp(i) = -simplifyFraction(sigma_e(h)/sey(h));

    elseif i==6; temp(i) = simplify(ex(h)*oz(h) - ox(h)*ez(h));
    elseif i==7; temp(i) = (ez(h)*Dox(h) - ex(h)*Doz(h));
    elseif i==8; temp(i) = (ox(h)*Doz(h) - oz(h)*Dox(h));
    elseif i==9; temp(i) = (ez(h)*Dex(h) - ex(h)*Dez(h));
    elseif i==10;temp(i) = (ox(h)*Dez(h) - oz(h)*Dex(h));
    end
%     disp([datestr(now, 'HH:MM:SS'),': ', num2str(i), '/10']);
end
dpo(h)   = temp(1);
dpe(h)   = temp(2);
Dpe(h)   = temp(3);
tau_o(h) = temp(4);
tau_e(h) = temp(5);
denom(h) = temp(6);
Ar00(h)  = temp(7);
Ar10(h)  = temp(8);
Ar01(h)  = temp(9);
Ar11(h)  = temp(10);


Ai(h) = [dpo(h), 0;
         0, dpe(h)+Dpe(h)];
Ar(h) = [-tau_o(h), 0;
         0,         -tau_e(h)] + ...
        1/denom(h) * [Ar00(h), Ar01(h);
                      Ar10(h), Ar11(h)];
                  
clearvars Dox(h) Doz(h) Dex(h) Dez(h)


%%% STAGE: Computing expansions
disp([datestr(now, 'HH:MM:SS'),': Computing expansions...']);

ar(h)  = ((subindexm(Ar(h), 1, 1) + subindexm(Ar(h), 2, 2)) / 2);
ai(h)  = ((subindexm(Ai(h), 1, 1) + subindexm(Ai(h), 2, 2)) / 2);
b1(h)  = ( subindexm(Ar(h), 1, 2));
b2(h)  = ( subindexm(Ar(h), 2, 1));
b3r(h) = ((subindexm(Ar(h), 1, 1) - subindexm(Ar(h), 2, 2)) / 2);
b3i(h) = ((subindexm(Ai(h), 1, 1) - subindexm(Ai(h), 2, 2)) / 2);
a_fun(h) = ar(h) + 1i*k*ai(h);
b3(h) = b3r(h) + 1i*k*b3i(h);

parfor (i=1:7, workers)
    if     i==1; temp(i) = (taylor(subindexm(Ar(h), 1, 1), h, 'Order', Order));
    elseif i==2; temp(i) = (taylor(subindexm(Ar(h), 2, 2), h, 'Order', Order));
    elseif i==3; temp(i) = (taylor(dpo,                    h, 'Order', Order));
    elseif i==4; temp(i) = (taylor(dpe,                    h, 'Order', Order));
    elseif i==5; temp(i) = (taylor(Dpe,                    h, 'Order', Order));
    elseif i==6; temp(i) = (taylor(subindexm(Ar(h), 1, 2), h, 'Order', Order));
    elseif i==7; temp(i) = (taylor(subindexm(Ar(h), 2, 1), h, 'Order', Order));
    end
    disp([datestr(now, 'HH:MM:SS'),': ', num2str(i), '/7']);
end
Tc1r(h) = temp(1);
Tc2r(h) = temp(2);
Tc1i(h) = temp(3);
Tdpe(h) = temp(4);
TDpe(h) = temp(5);
Tar(h)  = (Tc1r(h) + Tc2r(h))/2;
Tai(h)  = (Tc1i(h) + (Tdpe(h) + TDpe(h)))/2;
Tb3r(h) = (Tc1r(h) - Tc2r(h))/2;
Tb3i(h) = (Tc1i(h) - (Tdpe(h) + TDpe(h)))/2;
Tb1(h)  = temp(6);
Tb2(h)  = temp(7);
% Tb3(h)  = Tb3r(h) + 1i*k*Tb3i(h);


%%% STAGE: h1, h2 - Integrating
disp([datestr(now, 'HH:MM:SS'),': Integrating h1, h2...']);

syms h1r(h) h2r(h) h1i(h) h2i(h)

parfor (i=1:5, workers)
    if     i==1; temp(i) = int(Tc1r(y), y, 0, h);
    elseif i==2; temp(i) = int(Tc1i(y), y, 0, h);
    elseif i==3; temp(i) = int(Tc2r(y), y, 0, h);
    elseif i==4; temp(i) = int(Tdpe(y), y, 0, h);
    elseif i==5; temp(i) = int(TDpe(y), y, 0, h);
    end
    disp([datestr(now, 'HH:MM:SS'),': ', num2str(i), '/5']);
end
h1r(h) = temp(1);
h1i(h) = temp(2);
h2r(h) = temp(3);
h2i(h) = temp(4) + temp(5);


%%% STAGE: g1,g2 - Integrating
disp([datestr(now, 'HH:MM:SS'),': g1,g2 - Series expansion...']);

syms Ig1(h) Ig2(h) g_exp_i(h) Ig_Ga(h) Ig_Gb(h) Ga Gb erf0
syms mu_r(h) mu_i(h) mu_i_quad(h) Q1(h) Q2(h)

mu_r(h) = (h1r(h) - h2r(h))/2;
mu_i(h) = (h1i(h) - h2i(h))/2;

parfor (i=1:3, workers)
    if     i==1; temp(i) = taylor(exp(-2*mu_r(h))*Tb1(h),h,'Order',Order2);
    elseif i==2; temp(i) = taylor(exp( 2*mu_r(h))*Tb2(h),h,'Order',Order2);
    elseif i==3; temp(i) = taylor(mu_i(h),h,'Order',3);
    end
    disp([datestr(now, 'HH:MM:SS'),': ', num2str(i), '/3']);
end
Q1(h) = temp(1);
Q2(h) = temp(2);
mu_i_quad(h) = temp(3);

CE  = fliplr(coeffs(mu_i_quad(h),h,'All'));
CQ1 = fliplr(coeffs(Q1(h),h,'All'));
CQ2 = fliplr(coeffs(Q2(h),h,'All'));

Ig_ce1 = CE(2);
Ig_ce2 = 0;
if length(CE)==3
    disp([datestr(now, 'HH:MM:SS'),': g1,g2 - Gaussian integrals...']);
    
    Ig_ce2 = CE(3);
    parfor (i=1:2, workers)
        if     i==1; temp(i) = gaussian_integral(CQ1,Ga,Gb,h,erf0);
        elseif i==2; temp(i) = gaussian_integral(CQ2,Ga,Gb,h,erf0);
        end
    end
else
    disp([datestr(now, 'HH:MM:SS'),': g1,g2 - Exponential integrals...']);
    
    parfor (i=1:2, workers)
        if     i==1
            temp(i) = 0;
            for N=1:length(CQ1); temp(i) = temp(i) + CQ1(N) * int(y^(N-1) * exp(-Ga*y),y,0,h); end
        elseif i==2
            temp(i) = 0;
            for N=1:length(CQ2); temp(i) = temp(i) + CQ2(N) * int(y^(N-1) * exp(-Ga*y),y,0,h); end
        end
    end
end
Ig1(h) = temp(1);
Ig2(h) = temp(2);

clearvars mu_r(h) mu_i(h) mu_i_quad(h) Q1(h) Q2(h) CE CQ1 CQ2

%%% STAGE: Generating c routines
disp([datestr(now, 'HH:MM:SS'),': Generating c routines...']);

parfor (i=1:9, workers)
    if i==1
        gen_routine(StoreOutputAt,'a', ccode(sym(a)));
        gen_routine(StoreOutputAt,'b', ccode(sym(b)));
        gen_routine(StoreOutputAt,'c', ccode(sym(c)));
        gen_routine(StoreOutputAt,'eo',ccode(sym(eo)));
        gen_routine(StoreOutputAt,'ee',ccode(sym(ee)));
        gen_routine(StoreOutputAt,'sigma_o',ccode(sym(sigma_o)));
        gen_routine(StoreOutputAt,'sigma_e',ccode(sym(sigma_e)));
        write_version(StoreOutputAt, ProfileVersion);
        write_type(StoreOutputAt, type);
    elseif i==2;  gen_routine(StoreOutputAt,'Ig1',    ccode(sym(Ig1)));
    elseif i==3;  gen_routine(StoreOutputAt,'Ig2',    ccode(sym(Ig2)));
    elseif i==4;  gen_routine(StoreOutputAt,'h2i',    ccode(sym(h2i)));
    elseif i==5;  gen_routine(StoreOutputAt,'Ig_ce1', ccode(sym(Ig_ce1)));
    elseif i==6;  gen_routine(StoreOutputAt,'Ig_ce2', ccode(sym(Ig_ce2)));
    elseif i==7;  gen_routine(StoreOutputAt,'h1r',    ccode(sym(h1r)));
    elseif i==8;  gen_routine(StoreOutputAt,'h1i',    ccode(sym(h1i)));
    elseif i==9;  gen_routine(StoreOutputAt,'h2r',    ccode(sym(h2r)));
    end
    disp([datestr(now, 'HH:MM:SS'),': ', num2str(i), '/9']);
end

disp([datestr(now, 'HH:MM:SS'),': Done.']);

return;





Ig2vpa = vpa(Ig_ce2);
isGaussian = 1;
if isSymType(Ig2vpa,'constant') && double(Ig2vpa)==0
    isGaussian = 0;
end

g3r(h) = (h1r(h) - h2r(h))/2;
fr(h)  = (h1r(h) + h2r(h))/2;
g3i(h) = (h1i(h) - h2i(h))/2;
fi(h)  = (h1i(h) + h2i(h))/2;

g3(h)  = g3r(h) + 1i*k*g3i(h);
f(h)   = fr(h)  + 1i*k*fi(h);

if isGaussian == 1
    c1    = Ig_ce1;
    c2    = Ig_ce2;
    Ig_Ga = (1-1i)*sqrt(k) * c1 / sqrt(2*c2);
    Ig_Gb = (1-1i)*sqrt(k) * sqrt(2*c2);
    g_exp_i = 1i*k/2 * c1^2/c2;
    
    Ig1_erf0(h) = Ig_erf0(1i*Ig_Ga,1i*Ig_Gb,h);
    Ig2_erf0(h) = Ig_erf0(   Ig_Ga,   Ig_Gb,h);

    g1(h)  = sqrt(2*pi) * exp( (2*g3(h) + g_exp_i)) * subs(Ig1(h),{Ga,Gb,erf0},{1i*Ig_Ga,1i*Ig_Gb,Ig1_erf0});
    g2(h)  = sqrt(2*pi) * exp(-(2*g3(h) + g_exp_i)) * subs(Ig2(h),{Ga,Gb,erf0},{   Ig_Ga,   Ig_Gb,Ig2_erf0});
else
    Ga1(h) = 2i*k*Ig_ce1;
    Ga2(h) = -Ga1(h);

    g1(h)  = exp( 2*g3(h)) * subs(Ig1(h),Ga,Ga1);
    g2(h)  = exp(-2*g3(h)) * subs(Ig2(h),Ga,Ga2);
end

