function [Iro,Ito] = I_xo(K,eo,eo2,A,A2)
%function [Iro,Ito] = I_xo(K,eo,eo2,A,A2)
% Computes the Poynting vector of an ordinary reflection and refraction for a ray incident to
% an anistropic medium
% K is the incidence parameter
% eo is ordinary index-of-refraction of the source medium
% eo2 is ordinary index-of-refraction of the destination medium
% A is the optic axis of the source medium
% A2 is the optic axis of the destination medium

% Returns: Iro - the reflected ray, Ito - the refracted ray.


    [E_op, ~] = E_o(eo,A,K);
    qp = qo(eo,K);
    [~, E_om] = E_o(eo2,A2,K);
    qm = -qo(eo2,K);

    Iro = Poynting(qp,E_op,K);
    Ito = Poynting(qm,E_om,K);
    Iro = Iro / norm(Iro);
    Ito = Ito / norm(Ito);
end

