function [y] = SY(lambda)
%function [y] = SY(lambda)
% Curve fitted analytic approximation for the Y channel of CIE XYZ colour matching functions
% Takes wavelength in micrometre as input
%
% From:
% Chris Wyman, Peter pike Sloan, and Peter Shirley. 2013.
% Simple analytic approximations to the CIE XYZ color matching functions.
% In http://jcgt.org/published/0002/02/01/. 1–11.


    a0=0.821;
    a1=0.286;
    b0=568.8;
    b1=530.9;
    c0=0.0213;
    c1=0.0613;
    d0=0.0247;
    d1=0.0322;
    
    S = @(x,y,z) y.*(1-heaviside(x))+z.*heaviside(x);
    l = lambda*1000;
    
    N = 0.106943779063196;
    y = (a0 * exp(-.5 * ((l-b0) .* S(l-b0,c0,d0)).^2) + ...
         a1 * exp(-.5 * ((l-b1) .* S(l-b1,c1,d1)).^2)) / N;
end

