function [roo,roe,tos,top,reo,ree,tes,tep] = fresnel_aniso_iso(eo,ee,A,phi,eo2)
%function [roo,roe,tos,top,reo,ree,tes,tep] = fresnel_aniso_iso(eo,ee,A,phi,eo2)
% Computes the fresnel coefficients for an anisotropic-to-isotropic
% interface
% eo is the ordinary index-of-refraction of the source medium
% ee is the extraordinary index-of-refraction of the source medium
% A is the optic axis of the source medium
% phi is the angle of refraction in the destination medium
% eo2 is the index-of-refraction of the destination medium


    theta = phi;
    k = 1;
    K = k * eo2 * sin(theta);
    q2om = -qo(eo2,K);
    qop  =  qo(eo,K);
    [qep,qem] = qe(eo,ee,A,K);
    [Eop,Eom] = E_o(eo,A,K);
    [Eep,Eem] = E_e(eo,ee,A,K);
    
    Eop1 = Eop(1);
    Eop2 = Eop(2);
    Eop3 = Eop(3);
    Eep1 = Eep(1);
    Eep2 = Eep(2);
    Eep3 = Eep(3);
    Eom1 = Eom(1);
    Eom2 = Eom(2);
    Eom3 = Eom(3);
    Eem1 = Eem(1);
    Eem2 = Eem(2);
    Eem3 = Eem(3);

    B_1 = -Eep1*K*cos(theta)*(q2om - qep);
    B_2 = -Eom1*K*cos(theta)*(q2om + qop);
    B_3 = -Eop1*K*cos(theta)*(qop - q2om);
    D_1 = K*sin(theta) - q2om*cos(theta);
    D_2 = Eep3*(D_1 + qep*cos(theta));
    D_3 = Eem3*(D_1 + qem*cos(theta));
    D_4 = Eop3*(D_1 + qop*cos(theta));

    N2 = (Eep1*(qep - q2om)*D_4 + Eop1*(q2om - qop)*D_2 + Eop2*B_1 + Eep2*B_3);

    roo = -(Eep1*Eom3*(q2om - qep)*(qop*cos(theta) - D_1) + Eom1*(q2om + qop)*D_2 + Eom2*B_1 - Eep2*B_2) / N2;
    roe = -(-Eom1*Eop3*(qop*(qop*cos(theta) + K*sin(theta)) + q2om*D_1) + Eom3*Eop1*(qop*(qop*cos(theta) - K*sin(theta)) + q2om*D_1) + Eop2*B_2 + Eom2*B_3) / N2;
    tos = (Eep1*qep*roe - Eom1*qop + Eop1*qop*roo) / q2om;
    top = -(Eom3*qop - Eom2*K - (Eep2*K + Eep3*qep)*roe - (Eop2*K + Eop3*qop)*roo) / D_1;

    reo = (Eem1*(qem - q2om)*(D_2 + Eep2*K*cos(theta)) + Eep1*(q2om - qep)*(D_3 + Eem2*K*cos(theta))) / N2;
    ree = -(Eem1*(qem - q2om)*D_4 - Eem1*Eop2*K*cos(theta)*(q2om - qem) + Eem2*B_3 + Eop1*(q2om - qop)*D_3) / N2;
    tes = (Eem1*qem + Eep1*qep*ree + Eop1*qop*reo) / q2om;
    tep = -(Eem3 + Eep3*ree + Eop3*reo) / cos(theta);
end

