#version 450

#extension GL_ARB_separate_shader_objects : enable
#extension GL_ARB_shading_language_420pack : enable

layout (location = 0) in vec4 inPos;
layout (location = 1) in vec3 inNormal;
layout (location = 2) in vec2 inUV;
layout (location = 3) in vec3 inColor;

layout (binding = 0) uniform restrict UBO {
	mat4 projection;
	mat4 modelView;
	mat4 invModelView;
	vec4 cameraPosition;
	vec4 lightPos;
	vec4 lightRadii;
	float wavelength;
	float white_light;
} ubo;
layout (binding = 1) uniform restrict Params {
	float t;
	float exposure;
	float tau;
	float polarization;
} params;

layout (constant_id = 0) const float scale_x = 0;
layout (constant_id = 1) const float scale_y = 0;
layout (constant_id = 2) const float scale_z = 0;
layout (constant_id = 3) const float offset_x = 0;
layout (constant_id = 4) const float offset_y = 0;
layout (constant_id = 5) const float offset_z = 0;
layout (constant_id = 6) const int scale_with_tau = 0;

layout (location = 0) out vec3 outColor;
layout (location = 1) out vec3 outPos;
layout (location = 2) out vec3 outPosW;

void main() {
	const vec3 offset = vec3(offset_x,offset_y,offset_z);
	vec3 scale = vec3(scale_x,scale_y,scale_z);
	if (scale_with_tau==1)
		scale *= vec3(1,params.tau*2e-2f,1);
	if (scale_with_tau==2)
		scale *= vec3(params.tau * .5f);

	const vec3 P = inPos.xyz*scale + offset;
	const vec4 transformed = ubo.modelView * vec4(P, 1);

	outColor = inColor;
	outPos = transformed.xyz;
	outPosW = P;
	
	gl_Position = ubo.projection * transformed;	
}
