/*
*
*	Example implementation for "Accurate Rendering of Liquid-Crystals and Inhomogeneous Optically Anisotropic Media"
*
* This code is licensed under the MIT license (MIT) (http://opensource.org/licenses/MIT)
*/


#version 450

#extension GL_ARB_separate_shader_objects : enable
#extension GL_ARB_shading_language_420pack : enable

layout (constant_id = 0) const int white_light = 0;
layout (binding = 0) uniform restrict UBO {
	mat4 projection;
	mat4 modelView;
	mat4 invModelView;
	vec4 cameraPosition;
	vec4 lightPos;
	vec4 lightRadii;
	float wavelength;
	float white_light;
} ubo;
layout (binding = 1) uniform restrict Params {
	float t;
	float exposure;
	float tau;
	float polarization;
} params;

layout (location = 0) in vec3 color;
layout (location = 1) in vec3 position;
layout (location = 2) in vec3 wposition;
layout (location = 3) in vec3 normal;

layout (location = 0) out vec4 outFragColor;

const float pi = 3.1415f;


float sqr(float x) { return x*x; }
// Adobe XYZ to RGB
vec3 XYZtoRGB(vec3 XYZ) {
	return XYZ * mat3( 2.0413690, -0.5649464, -0.3446944,
					  -0.9692660,  1.8760108,  0.0415560,
					   0.0134474, -0.1183897,  1.0154096);
}
vec3 white_point() { return vec3(0.9504, 1.0000, 1.0888); }

// Inputs:  Wavelength in nanometers
// From "Simple Analytic Approximations to the CIE XYZ Color Matching Functions"
float X_1931(float wave) {
	const float t1 = (wave-442.0f) * ((wave<442.0f)?0.0624f:0.0374f);
	const float t2 = (wave-599.8f) * ((wave<599.8f)?0.0264f:0.0323f);
	const float t3 = (wave-501.1f) * ((wave<501.1f)?0.0490f:0.0382f);
	return 0.362f * exp(-0.5f * sqr(t1)) + 1.056f * exp(-0.5f * sqr(t2)) - 0.065f * exp(-0.5f * sqr(t3));
}
float Y_1931(float wave) {
	const float t1 = (wave-568.8f) * ((wave<568.8f)?0.0213f:0.0247f);
	const float t2 = (wave-530.9f) * ((wave<530.9f)?0.0613f:0.0322f);
	return 0.821f * exp(-0.5f * sqr(t1)) + 0.286f * exp(-0.5f * sqr(t2));
}
float Z_1931(float wave) {
	const float t1 = (wave-437.0f) * ((wave<437.0f)?0.0845f:0.0278f);
	const float t2 = (wave-459.0f) * ((wave<459.0f)?0.0385f:0.0725f);
	return 1.217f * exp(-0.5f * sqr(t1)) + 0.681f * exp(-0.5f * sqr(t2));
}


vec3 materialcolor()
{
	return vec3(0.7,0.6,0.5);
}

void main() {
	const vec3 o = normalize(-position * mat3(ubo.invModelView));
	
	const float wavelength = ubo.wavelength;
	const vec3 lightColor = XYZtoRGB(ubo.white_light>0 ? white_point() : vec3(X_1931(wavelength), Y_1931(wavelength), Z_1931(wavelength)));

	vec3 c = mix(lightColor,vec3(1),0.5) * (sin(wposition.x+wposition.y+wposition.z)*0.1f + 0.5f) * 0.002f;

	outFragColor = vec4(c,1);
}
