function [E_ep, E_em] = E_e(eo,ee,A,K)
% function [E_ep, E_em] = E_e(eo,ee,A,K)  
% Computes the extraordinary electric field for the positive and negative normal
% modes
% eo is ordinary index-of-refraction
% ee is extraordinary index-of-refraction
% A is the optic axis
% K is the incidence parameter
    
    [qep,qem] = qe(eo,ee,A,K);
    Wep = [0,qep,-K];
    Wem = [0,qem,-K];
    AWep = dot(A,Wep);
    AWem = dot(A,Wem);
    
    E_ep = (eo^2*A - AWep*Wep) / sqrt(eo^4 + AWep^2*(qep^2 + K^2 - 2*eo^2));
    E_em = (eo^2*A - AWem*Wem) / sqrt(eo^4 + AWem^2*(qem^2 + K^2 - 2*eo^2));
end

