function [Ire,Ite,W_r,W_t,er,et] = I_xe(K,eo,eo2,ee,ee2,A,A2)
%function [Ire,Ite,W_r,W_t,er,et] = I_xe(K,eo,eo2,ee,ee2,A,A2)
% Computes the Poynting vectors of an extraordinary reflection and refraction for a ray 
% incident to an anisotropic medium
% K is the incidence parameter
% eo is ordinary index-of-refraction of the source medium
% eo2 is ordinary index-of-refraction of the destination medium
% ee is extraordinary index-of-refraction of the source medium
% ee2 is extraordinary index-of-refraction of the destination medium
% A is the optic axis of the source medium
% A2 is the optic axis of the destination medium
% Setting eo=ee indicates an isotropic source medium, in which case A is ignored.
%
% Returns: Ire - the reflected ray, Ite - the refracted ray, W_r - the reflected wave's direction
% of propagation, W_t - the refracted wave's direction of propagation, er - the effective 
% index-of-refraction of the reflected wave, et - the effective index-of-refraction of the
% refracted wave.


    [E_ep, ~] = E_e(eo,ee,A,K);
    [qep, ~] = qe(eo,ee,A,K);
    [~, E_em] = E_e(eo2,ee2,A2,K);
    [~, qem] = qe(eo2,ee2,A2,K);

    Ire = Poynting(qep,E_ep,K);
    Ite = Poynting(qem,E_em,K);
    Ire = Ire / norm(Ire);
    Ite = Ite / norm(Ite);
    
    W_r = [0,qep,-K];
    W_t = [0,qem,-K];
    W_r = W_r / norm(W_r);
    W_t = W_t / norm(W_t);
    
    er = K/sqrt(1-W_r(2)^2);
    et = K/sqrt(1-W_t(2)^2);
end
