function [x] = SX(lambda)
%function [x] = SX(lambda)
% Curve fitted analytic approximation for the X channel of CIE XYZ colour matching functions
% Takes wavelength in micrometre as input
%
% From:
% Chris Wyman, Peter pike Sloan, and Peter Shirley. 2013.
% Simple analytic approximations to the CIE XYZ color matching functions. 
% In http://jcgt.org/published/0002/02/01/. 1–11.


    a0=0.362;
    a1=1.056;
    a2=-0.065;
    b0=442.0;
    b1=599.8;
    b2=501.1;
    c0=0.0624;
    c1=0.0264;
    c2=0.0490;
    d0=0.0374;
    d1=0.0323;
    d2=0.0382;
    
    S = @(x,y,z) y.*(1-heaviside(x))+z.*heaviside(x);
    l = lambda*1000;
    
    x0 = a0 * exp(-.5 * ((l-b0) .* S(l-b0,c0,d0)).^2);
    x1 = a1 * exp(-.5 * ((l-b1) .* S(l-b1,c1,d1)).^2);
    x2 = a2 * exp(-.5 * ((l-b2) .* S(l-b2,c2,d2)).^2);
    
    N = 0.106713659780474;
    x = (x0+x1+x2) / N;
end

