function [z] = SZ(lambda)
%function [z] = SZ(lambda)
% Curve fitted analytic approximation for the Z channel of CIE XYZ colour matching functions
% Takes wavelength in micrometre as input
%
% From:
% Chris Wyman, Peter pike Sloan, and Peter Shirley. 2013.
% Simple analytic approximations to the CIE XYZ color matching functions.
% In http://jcgt.org/published/0002/02/01/. 1–11.


    a0=1.217;
    a1=0.681;
    b0=437.0;
    b1=459.0;
    c0=0.0845;
    c1=0.0385;
    d0=0.0278;
    d1=0.0725;
    
    S = @(x,y,z) y.*(1-heaviside(x))+z.*heaviside(x);
    l = lambda*1000;
    
    N = 0.106806301192528;
    z = (a0 * exp(-.5 * ((l-b0) .* S(l-b0,c0,d0)).^2) + ...
         a1 * exp(-.5 * ((l-b1) .* S(l-b1,c1,d1)).^2)) / N;
end

