function [rss, rsp, tso, tse, rps, rpp, tpo, tpe] = fresnel_iso_aniso(ei,theta,eo,ee,A)
%function [rss, rsp, tso, tse, rps, rpp, tpo, tpe] = fresnel_iso_aniso(ei,theta,eo,ee,A)
% Computes the fresnel coefficients for an isotropic-to-anisotropic
% interface
% ei is the index-of-refraction perceived by the incident wave
% theta is incidence angle
% eo is the ordinary index-of-refraction of the destination medium
% ee is the extraordinary index-of-refraction of the destination medium
% A is the optic axis of the destination medium


    k = 1;
    K = k*ei*sin(theta);
    qi = -qo(ei,K);
    [~,E2om]=E_o(eo,A,K);
    [~,E2em]=E_e(eo,ee,A,K);
    q2om = -qo(eo,K);
    [~, q2em] = qe(eo,ee,A,K);
    
    E2om1 = E2om(1);
    E2om2 = E2om(2);
    E2om3 = E2om(3);
    E2em1 = E2em(1);
    E2em2 = E2em(2);
    E2em3 = E2em(3);

    A_1 = -E2em1*E2om2*K*cos(theta);
    A_2 = -E2em2*E2om1*K*cos(theta);
    C_1 = qi*cos(theta) - K*sin(theta);
    C_2 = E2em1*E2om3*(C_1 + q2om*cos(theta)) - A_1;
    C_3 = E2em3*E2om1*(C_1 + q2em*cos(theta)) - A_2;

    Ns = C_2*(qi + q2em) - C_3*(qi + q2om);

    rps = (2*E2em1*E2om1*cos(theta)*C_1*(q2om - q2em)) / Ns;
    rpp = (-E2em1*E2om3*(C_1 - q2om*cos(theta))*(qi + q2em) + E2em3*E2om1*(C_1 - q2em*cos(theta))*(qi + q2om) - A_1*(q2em + qi) + A_2*(q2om + qi)) / Ns;
    tpo = -(2*E2em1*cos(theta)*(q2em + qi)*C_1) / Ns;
    tpe = (2*E2om1*cos(theta)*(q2om + qi)*C_1) / Ns;

    rss = (C_2*(qi - q2em) + C_3*(q2om - qi)) / Ns;
    rsp = (2*qi*(E2em3*E2om2*K - E2em2*E2om3*K + E2em3*E2om3*(q2om - q2em))) / Ns;
    tso = -(2*qi*(E2em2*K*cos(theta) + E2em3*C_1 + E2em3*q2em*cos(theta))) / Ns;
    tse = (2*qi*(E2om2*K*cos(theta) + E2om3*C_1 + E2om3*q2om*cos(theta))) / Ns;
end

